using System;
using System.Net;
using System.IO;
using System.Text;
using System.Xml.Serialization;

namespace AladdinOpenAPI
{
    public abstract class ItemRequest
    {
        public AladdinService Service;

        public ItemCover Cover = ItemCover.Mid;
        public ResponseOutput Output = ResponseOutput.XML;
        public string Partner = string.Empty;
        public bool Omitkey = false;
        public int Version = 20070901;
        public ItemOperationType OperationType;

        private string _callBack = string.Empty;
        public string CallBack
        {
            set 
            {
                if(this.Output!=ResponseOutput.JS)
                    throw new OpenAPIException("CallBack은 Output이 JS일때에만 가능합니다.");

                this._callBack = value;
            }

            get
            {
                return this._callBack;
            }
        }

        public string TTBKey
        {
            get { return Service.TTBKey; }
        }

        private ItemRequest() { }

        internal ItemRequest(AladdinService service)
        {
            this.Service = service;
        }
    
        protected string GetOmitkey(){
            return this.Omitkey ? "1" : string.Empty;
        }

        public ItemResponse GetItemResponse()
        {
            string url = GetOperationURL();
            string query = GetQuery();

            HttpWebRequest req = null;
            HttpWebResponse res = null;
            Stream reqStream = null;
            StreamWriter sw = null;
            Stream resStream = null;
            StreamReader sr = null;
            TextReader txtReader = null;
            ItemResponse response = null;
            string resultMsg = string.Empty;

            try
            {
                req = (HttpWebRequest)HttpWebRequest.Create(url);
                req.Method = "POST";
                req.ContentType = "application/x-www-form-urlencoded";
                reqStream = req.GetRequestStream();
                sw = new StreamWriter(reqStream);
                sw.Write(query);
                sw.Close();
                reqStream.Close();

                res = (HttpWebResponse)req.GetResponse();
                resStream = res.GetResponseStream();
                sr = new StreamReader(resStream);
                resultMsg = sr.ReadToEnd();
                sr.Close();
                resStream.Close();
                res.Close();

                if(resultMsg.IndexOf("<error xmlns=")>-1)
                {
                    throw new OpenAPIException(ErrorMessageCapture.GetErrorMessage(resultMsg));
                }

                XmlSerializer serializer = new XmlSerializer(typeof(AladdinOpenAPI.ItemResponse));
                txtReader = new StringReader(resultMsg);
                response = (AladdinOpenAPI.ItemResponse)serializer.Deserialize(txtReader);
                txtReader.Close();
                return response;
            }
            catch (OpenAPIException apiEx)
            {
                // Console.WriteLine(apiEx.ToString());
                // Console.WriteLine(resultMsg);
                throw apiEx;
            }
            catch (Exception ex)
            {
                // Console.WriteLine(ex.ToString());
                // Console.WriteLine(resultMsg);
                throw ex;
            }
            finally
            {
                if (res != null) try { res.Close(); }
                    catch (Exception) { };
                if (reqStream != null) try { reqStream.Close(); }
                    catch (Exception) { };
                if (sw != null) try { sw.Close(); }
                    catch (Exception) { };
                if (resStream != null) try { resStream.Close(); }
                    catch (Exception) { };
                if (sw != null) try { sw.Close(); }
                    catch (Exception) { };
                if (txtReader != null) try { txtReader.Close(); }
                    catch (Exception) { };
            }
        }

        public abstract string GetOperationURL();
        public abstract string GetQuery();
    }
}